@@ -1,6 +1,6 @@
 ##---------------------------------------------------------------------------##
 ##  File:
-##      @(#) mhopt.pl 2.20 01/04/10 21:36:41
+##      @(#) mhopt.pl 2.21 01/08/26 02:25:24
 ##  Author:
 ##      Earl Hood       mhonarc@pobox.com
 ##  Description:
@@ -63,6 +63,7 @@ sub get_resources {
 	"expireage=i",	# Time in seconds from current if message expires
 	"folrefs",	# Print links to explicit follow-ups/references
 	"footer=s",	# File containing user text for bottom of index page
+			# 	(option no longer applicable)
 	"force",	# Perform archive operation even if unable to lock
 	"fromfields=s", # Fields that contains the "from" of a message
 	"genidx",	# Generate an index based upon archive contents
@@ -71,6 +72,7 @@ sub get_resources {
 	"gzipfiles",	# Gzip files
 	"gziplinks",	# Add ".gz" extensions to files
 	"header=s",	# File containing user text for top of index page
+			# 	(option no longer applicable)
 	"htmlext=s",	# Extension for HTML files
 	"idxfname=s",	# Filename of index page
 	"idxprefix=s",	# Filename prefix for multi-page main index
@@ -198,7 +200,9 @@ sub get_resources {
 
     ## Check std{in,out,err} options
     DUP: {
-	$MhaStdin = \*STDIN;
+	$MhaStdin  = \*STDIN;
+	#$MhaStdout = \*STDOUT;
+	#$MhaStderr = \*STDERR;
 	STDOUTERR: {
 	    if (defined($opt{'stdout'}) && !ref($opt{'stdout'})) {
 		open(STDOUT, ">>$opt{'stdout'}") ||
@@ -276,26 +280,33 @@ sub get_resources {
     require 'mhrmm.pl'  	if $RMM;
     require 'mhnote.pl'  	if $ANNOTATE;
 
+    print STDOUT "This is MHonArc v$VERSION, Perl $]\n";
+
     ## Evaluate site local initialization
     delete($INC{'mhasiteinit.pl'});      # force re-evaluation
     eval { require 'mhasiteinit.pl'; };  # ignore status
 
     ## Read default resource file
-    if ($DefRcFile) {
-	&read_fmt_file($DefRcFile);
-    } else {
-	$tmp = join($DIRSEP, $ENV{'HOME'}, $DefRcName);
-	if (! -e $tmp) {
-	    local $_;
-	    foreach (@INC) {
-		if (-e join($DIRSEP, $_, 'mhamain.pl')) {
-		    $tmp = join($DIRSEP, $_, $DefRcName);
-		    last;
-		}
+    DEFRCFILE: {
+	if ($DefRcFile) {
+	    read_fmt_file($DefRcFile);
+	    last DEFRCFILE;
+	}
+	if (defined $ENV{'HOME'}) {
+	    # check if in home directory
+	    $tmp = join($DIRSEP, $ENV{'HOME'}, $DefRcName);
+	    if (-e $tmp) {
+		read_fmt_file($tmp);
+		last DEFRCFILE;
 	    }
 	}
-	if (-e $tmp) {
-	    &read_fmt_file($tmp);
+	local $_;
+	foreach (@INC) {
+	    $tmp = join($DIRSEP, $_, $DefRcName);
+	    if (-e $tmp) {
+		read_fmt_file($tmp);
+		last DEFRCFILE;
+	    }
 	}
     }
 
@@ -356,9 +367,16 @@ sub get_resources {
 
 	    ## Check for 1.x archive, and update data as needed
 	    if ($DbVERSION =~ /^1\./) {
-		print STDOUT "Updating database data to 2.0 ...\n"
+		print STDOUT "Updating database $DbVERSION data ...\n"
 		    unless $QUIET;
 		&update_data_1_to_2();
+		&update_data_2_4_to_later();
+	    }
+	    ## Check for 2.[0-4] archive
+	    if ($DbVERSION =~ /^2\.[0-4]\./) {
+		print STDOUT "Updating database $DbVERSION data ...\n"
+		    unless $QUIET;
+		&update_data_2_4_to_later();
 	    }
 
 	    ## Set %Follow here just incase it does not get recomputed
@@ -413,8 +431,8 @@ sub get_resources {
 	unshift(@PerlINC, @array);
     }
 
-    &remove_dups(*OtherIdxs);
-    &remove_dups(*PerlINC);
+    @OtherIdx = remove_dups(\@OtherIdxs);
+    @PerlINC  = remove_dups(\@PerlINC);
 
     ## Require mail parsing library
     unshift(@INC, @PerlINC);
@@ -430,9 +448,7 @@ sub get_resources {
     $DBPathName = join($DIRSEP, $OUTDIR, $DBFILE);
 
     $DOCURL	= $opt{'docurl'}     if $opt{'docurl'};
-    $FOOTER	= $opt{'footer'}     if $opt{'footer'};
     $FROM	= $opt{'msgsep'}     if $opt{'msgsep'};
-    $HEADER	= $opt{'header'}     if $opt{'header'};
     $IDXPREFIX	= $opt{'idxprefix'}  if $opt{'idxprefix'};
     $IDXSIZE	= $opt{'idxsize'}    if defined($opt{'idxsize'});
 	$IDXSIZE *= -1  if $IDXSIZE < 0;
@@ -447,7 +463,8 @@ sub get_resources {
     $TTITLE	= $opt{'ttitle'}     if $opt{'ttitle'};
     $MsgPrefix	= $opt{'msgprefix'}  if defined($opt{'msgprefix'});
     $GzipExe	= $opt{'gzipexe'}    if $opt{'gzipexe'};
-    $VarExp	= $opt{'varregex'}   if $opt{'varregex'};
+    $VarExp	= $opt{'varregex'}   if $opt{'varregex'} &&
+				        ($opt{'varregex'} =~ /\S/);
 
     $IDXNAME	= $opt{'idxfname'} || $IDXNAME || $ENV{'M2H_IDXFNAME'} ||
 		  "maillist.$HtmlExt";
@@ -540,8 +557,8 @@ sub get_resources {
     @FromFields	 = split(/:/, $opt{'fromfields'})  if $opt{'fromfields'};
     foreach (@FromFields) { s/\s//g; tr/A-Z/a-z/; }
 
-    ($TSliceNBefore, $TSliceNAfter) = split(/:/, $opt{'tslice'})
-	if $opt{'tslice'};
+    ($TSliceNBefore, $TSliceNAfter, $TSliceInclusive) =
+	split(/[:;]/, $opt{'tslice'})  if $opt{'tslice'};
 
     @Months   = split(/:/, $opt{'months'}) 	if defined($opt{'months'});
     @months   = split(/:/, $opt{'monthsabr'})  	if defined($opt{'monthsabr'});
@@ -673,7 +690,7 @@ sub usage {
 ##
 sub read_fmt_file {
     require 'mhrcfile.pl';
-    &read_resource_file($_[0]);
+    &read_resource_file;  # implicit passing of @_
 }
 
 ##---------------------------------------------------------------------------
@@ -703,9 +720,28 @@ sub update_data_1_to_2 {
 	$Subject{$index} =~ s/\&([\w-.]+);/&entname_to_char($1)/ge;
     }
     delete $IndexNum{''};
-    $TLITXT = '<LI>' . $TLITXT  unless ($TLITXT) && ($TLITXT =~ /<li>/i);
-    $THEAD .= "<UL>\n"   unless ($THEAD) && ($THEAD =~ m%<ul>\s*$%i);
-    $TFOOT  = "</UL>\n"  unless ($TFOOT) && ($TFOOT =~ m%^\s*</ul>%i);
+    $TLITXT = '<li>' . $TLITXT  unless ($TLITXT) && ($TLITXT =~ /<li>/i);
+    $THEAD .= "<ul>\n"   unless ($THEAD) && ($THEAD =~ m%<ul>\s*$%i);
+    $TFOOT  = "</ul>\n"  unless ($TFOOT) && ($TFOOT =~ m%^\s*</ul>%i);
+}
+
+##---------------------------------------------------------------------------
+##	Update 2.4, or earlier, data.
+##
+sub update_data_2_4_to_later {
+    my($index, $value);
+    while (($index, $value) = each(%Refs)) {
+	next  if ref($value);
+	$Refs{$index} = [ split(/$X/o, $value) ];
+    }
+    while (($index, $value) = each(%FollowOld)) {
+	next  if ref($value);
+	$FollowOld{$index} = [ split(/$bs/o, $value) ];
+    }
+    while (($index, $value) = each(%Derived)) {
+	next  if ref($value);
+	$Derived{$index} = [ split(/$X/o, $value) ];
+    }
 }
 
 ##---------------------------------------------------------------------------
