@@ -1,13 +1,13 @@
 ##---------------------------------------------------------------------------##
 ##  File:
-##      @(#) mhthread.pl 2.4 98/10/10 16:29:57
+##      @(#) mhthread.pl 2.7 01/08/26 02:25:26
 ##  Author:
-##      Earl Hood       earlhood@usa.net
+##      Earl Hood       mhonarc@mhonarc.org
 ##  Description:
 ##      Thread routines for MHonArc
 ##---------------------------------------------------------------------------##
 ##    MHonArc -- Internet mail-to-HTML converter
-##    Copyright (C) 1995-1998	Earl Hood, earlhood@usa.net
+##    Copyright (C) 1995-2001	Earl Hood, mhonarc@mhonarc.org
 ##
 ##    This program is free software; you can redistribute it and/or modify
 ##    it under the terms of the GNU General Public License as published by
@@ -93,6 +93,9 @@ sub write_thread_index {
 	}
 	print STDOUT "Writing $TIDXPATHNAME ...\n"  unless $QUIET;
 
+	($tmpl = $SSMARKUP) =~ s/$VarExp/&replace_li_var($1,'')/geo;
+	print $handle $tmpl;
+
 	print $handle "<!-- ", &commentize("MHonArc v$VERSION"), " -->\n";
 
 	($tmpl = $TIDXPGBEG) =~ s/$VarExp/&replace_li_var($1,'')/geo;
@@ -211,19 +214,22 @@ sub compute_threads {
     %SReplies 	 = ();
 
     ##	Sort by date first for subject based threads
-    @ThreadList = sort increase_index keys %Subject;
+    @ThreadList = sort_messages(0,0,0,0);
 
     ##	Find first occurrances of subjects
-    foreach $index (@ThreadList) {
-	$tmp = lc $Subject{$index};
-	1 while (($tmp =~ s/^$SubReplyRxp//io) ||
-		 ($tmp =~ s/\s*-\s*re(ply|sponse)\s*$//io));
+    if (!$NoSubjectThreads) {
+	foreach $index (@ThreadList) {
+	    $tmp = lc $Subject{$index};
+	    1 while (($tmp =~ s/^$SubReplyRxp//io) ||
+		     ($tmp =~ s/\s*-\s*re(ply|sponse)\s*$//io));
 
-	$stripsub{$index} = $tmp;
-	$FirstSub2Index{$tmp} = $index
-	    unless defined($FirstSub2Index{$tmp}) ||
-		   (defined($Refs{$index}) &&
-		    grep($MsgId{$_}, split(/$X/o, $Refs{$index})));
+	    $stripsub{$index} = $tmp;
+	    next  unless $tmp =~ /\S/;
+	    $FirstSub2Index{$tmp} = $index
+		unless defined($FirstSub2Index{$tmp}) ||
+		       (defined($Refs{$index}) &&
+			grep($MsgId{$_}, @{$Refs{$index}}));
+	}
     }
 
     ##	Compute thread data
@@ -231,7 +237,7 @@ sub compute_threads {
 	next  unless defined($Refs{$index});
 
 	# Check for explicit threading
-	if (@refs = split(/$X/o, $Refs{$index})) {
+	if (@refs = @{$Refs{$index}}) {
 	    $depth = 0;
 	    while ($msgid = pop(@refs)) {
 		if (($refindex = $MsgId{$msgid})) {
@@ -239,9 +245,9 @@ sub compute_threads {
 		    $HasRef{$index} = $refindex;
 		    $HasRefDepth{$index} = $depth;
 		    if ($Replies{$refindex}) {
-			$Replies{$refindex} .= $bs . $index;
+			push(@{$Replies{$refindex}}, $index);
 		    } else {
-			$Replies{$refindex} = $index;
+			$Replies{$refindex} = [ $index ];
 		    }
 		    next TCOMP;
 		}
@@ -251,23 +257,23 @@ sub compute_threads {
 
     } continue {
 	# Check for subject-based threading
-	if (!$HasRef{$index}) {
-	    if (($refindex = $FirstSub2Index{$stripsub{$index}}) &&
-		($refindex ne $index)) {
+	if (!$NoSubjectThreads && !$HasRef{$index}) {
+	    $refindex = $FirstSub2Index{$stripsub{$index}};
+	    if ($refindex && ($refindex ne $index)) {
 
 		$HasRef{$index} = $refindex;
 		$HasRefDepth{$index} = 0;
 		if ($SReplies{$refindex}) {
-		    $SReplies{$refindex} .= $bs . $index;
+		    push(@{$SReplies{$refindex}}, $index);
 		} else {
-		    $SReplies{$refindex} = $index;
+		    $SReplies{$refindex} = [ $index ];
 		}
 	    }
 	}
     }
 
     ## Calculate thread listing order
-    @ThreadList = &t_sort_messages();
+    @ThreadList = sort_messages($TNOSORT, $TSUBSORT, 0, $TREVERSE);
     foreach $index (@ThreadList) {
 	unless ($Counted{$index} || $HasRef{$index}) {
 	    &do_thread($index, 0);
@@ -286,9 +292,9 @@ sub do_thread {
     local(@repls, @srepls) = ();
 
     ## Get replies
-    @repls  = sort increase_index split(/$bs/o, $Replies{$idx})
+    @repls  = sort increase_index @{$Replies{$idx}}
 	if defined($Replies{$idx});
-    @srepls = sort increase_index split(/$bs/o, $SReplies{$idx})
+    @srepls = sort increase_index @{$SReplies{$idx}}
 	if defined($SReplies{$idx});
 
     ## Add index to printed order list (IMPORTANT SIDE-EFFECT)
@@ -317,13 +323,14 @@ sub do_thread {
 ##
 sub print_thread {
     local($handle, $idx, $top) = ($_[0], $_[1], $_[2]);
-    local(@repls, @srepls) = ();
-    local($attop, $haverepls, $hvnirepls, $single, $depth, $i);
+    my(@repls, @srepls) = ();
+    my($attop, $haverepls, $hvnirepls, $single, $depth, $i);
+    my $didtliend = 0;
 
     ## Get replies
-    @repls  = sort increase_index split(/$bs/o, $Replies{$idx})
+    @repls  = sort increase_index @{$Replies{$idx}}
 	if defined($Replies{$idx});
-    @srepls = sort increase_index split(/$bs/o, $SReplies{$idx})
+    @srepls = sort increase_index @{$SReplies{$idx}}
 	if defined($SReplies{$idx});
     $depth  = $HasRefDepth{$idx};
     $hvnirepls = (@repls || @srepls);
@@ -345,25 +352,29 @@ sub print_thread {
     $single  = ($top && !$hvnirepls);
 
     if ($attop) {
-	&print_thread_var($handle, $idx, *TTOPBEG);
+	&print_thread_var($handle, $idx, \$TTOPBEG);
     } elsif ($single) {
-	&print_thread_var($handle, $idx, *TSINGLETXT);
+	&print_thread_var($handle, $idx, \$TSINGLETXT);
     } else {
 	## Check for missing messages
 	if ($DoMissingMsgs) {
-	    for ($i = $depth; $i > 0; $i--) {
-		$level++;
-		&print_thread_var($handle, $idx, *TLINONE);
-		&print_thread_var($handle, $idx, *TSUBLISTBEG)
+	    for ($i=$depth; $i > 0; --$i) {
+		++$level;
+		&print_thread_var($handle, $idx, \$TLINONE);
+		&print_thread_var($handle, $idx, \$TSUBLISTBEG)
 		    if $level <= $TLEVELS;
 	    }
 	}
-	&print_thread_var($handle, $idx, *TLITXT);
+	&print_thread_var($handle, $idx, \$TLITXT);
     }
 
     ## Increment level count if their are replies
-    if ($haverepls) {
-	$level++;
+    ++$level  if ($haverepls);
+
+    ## Print list item close if hit max depth
+    if (!$attop && !$single && ($level > $TLEVELS)) {
+	&print_thread_var($handle, $idx, \$TLIEND);
+	$didtliend = 1;
     }
 
     ## Mark message printed
@@ -371,50 +382,48 @@ sub print_thread {
 
     ## Print sub-threads
     if (scalar(@repls) || scalar(@srepls)) {
-	&print_thread_var($handle, $idx, *TSUBLISTBEG)  if $level <= $TLEVELS;
+	&print_thread_var($handle, $idx, \$TSUBLISTBEG)  if $level <= $TLEVELS;
 	foreach (@repls) {
 	    &print_thread($handle, $_);
 	}
 	if (@srepls) {
-	    &print_thread_var($handle, $idx, *TSUBJECTBEG);
+	    &print_thread_var($handle, $idx, \$TSUBJECTBEG);
 	    foreach (@srepls) {
 		&print_thread($handle, $_);
 	    }
-	    &print_thread_var($handle, $idx, *TSUBJECTEND);
+	    &print_thread_var($handle, $idx, \$TSUBJECTEND);
 	}
-	&print_thread_var($handle, $idx, *TSUBLISTEND)  if $level <= $TLEVELS;
+	&print_thread_var($handle, $idx, \$TSUBLISTEND)  if $level <= $TLEVELS;
     }
 
     ## Decrement level count if their were replies
-    if ($haverepls) {
-	$level--;
-    }
+    --$level  if ($haverepls);
+
     ## Check for missing messages
     if ($DoMissingMsgs && !($attop || $single)) {
-	for ($i = $depth; $i > 0; $i--) {
-	    &print_thread_var($handle, $idx, *TLINONEEND);
-	    &print_thread_var($handle, $idx, *TSUBLISTEND)
+	for ($i=$depth; $i > 0; --$i) {
+	    &print_thread_var($handle, $idx, \$TLINONEEND);
+	    &print_thread_var($handle, $idx, \$TSUBLISTEND)
 		if $level <= $TLEVELS;
-	    $level--;
+	    --$level;
 	}
     }
 
     ## Close entry text
     if ($attop) {
-	&print_thread_var($handle, $idx, *TTOPEND);
-    } elsif (!$single) {
-	&print_thread_var($handle, $idx, *TLIEND);
+	&print_thread_var($handle, $idx, \$TTOPEND);
+    } elsif (!$single && !$didtliend) {
+	&print_thread_var($handle, $idx, \$TLIEND);
     }
 }
 
 ##---------------------------------------------------------------------------
-##	Print out text based upon resource variable referenced by *tvar.
+##	Print out text based upon resource variable referenced by $tvar.
 ##
 sub print_thread_var {
-    local($handle, $index, *tvar) = @_;
-    local($i_p0, $filename, $tmpl, $msgnum);
-
-    ($tmpl = $tvar) =~ s/$VarExp/&replace_li_var($1,$index)/geo;
+    my($handle, $index, $tvar) = @_;
+    my($tmpl);
+    ($tmpl = $$tvar) =~ s/$VarExp/&replace_li_var($1,$index)/geo;
     print $handle $tmpl;
 }
 
@@ -429,20 +438,39 @@ sub print_thread_var {
 ##	Returns string containing thread slice text.
 ##
 sub make_thread_slice {
-    local($refindex, $bcnt, $acnt) = @_;
-    local($slicetxt) = "";
+    my($refindex, $bcnt, $acnt, $inclusive) = @_;
+    my($slicetxt) = "";
 
-    local($pos)   = $Index2TLoc{$refindex};
-    local($start) = $pos - $bcnt;
-    local($end)   = $pos + $acnt;
-	  $start  = 0		  if $start < 0;
-	  $end    = $#TListOrder  if $end > $#TListOrder;
-    local(@a)	  = @TListOrder[$start..$end];
+    my($pos)   = $Index2TLoc{$refindex};
+    my($start) = $pos - $bcnt;
+    my($end)   = $pos + $acnt;
+    $start     = 0             if $start < 0;
+    $end       = $#TListOrder  if $end > $#TListOrder;
+    if ($inclusive) {
+	# adjust before count
+	if ($bcnt == 0 || $ThreadLevel{$TListOrder[$pos]} <= 0) {
+	    $start = $pos;
+	} else {
+	    for ($i=$pos-1; ($i > $start) && ($i > 0); --$i) {
+		last  if ($ThreadLevel{$TListOrder[$i]} <= 0);
+	    }
+	    $start = $i;
+	}
+	# adjust after count
+	if ($acnt != 0) {
+	    for ($i=$pos+1; ($i <= $end) && ($i <= $#TListOrder); ++$i) {
+		last  if ($ThreadLevel{$TListOrder[$i]} <= 0);
+	    }
+	    $end = $i-1;
+	}
 
-    local($lastlevel) = $ThreadLevel{$a[0]};
-    local($level)     = 0;  	## !!!Used in print_thread!!!
-    local(%Printed);		## !!!Used in print_thread!!!
-    local($tmpl, $index, $tlevel, $iscont, $i);
+    }
+    my(@a)         = @TListOrder[$start..$end];
+    my($lastlevel) = $ThreadLevel{$a[0]};
+    my($tmpl, $index, $tlevel, $iscont, $i);
+
+    local($level)     = 0;  	## !!!Used in make_thread!!!
+    local(%Printed)   = ();	## !!!Used in make_thread!!!
 
     ($tmpl = $TSLICEBEG) =~ s/$VarExp/&replace_li_var($1,'')/geo;
     $slicetxt .= $tmpl;
@@ -452,14 +480,14 @@ sub make_thread_slice {
 
     # check if continuing a thread
     if ($lastlevel > 0) {
-	($tmpl = $TCONTBEG) =~ s/$VarExp/&replace_li_var($1,$a[0])/geo;
+	($tmpl = $TSLICECONTBEG) =~ s/$VarExp/&replace_li_var($1,$a[0])/geo;
 	$slicetxt .= $tmpl;
     }
     # perform any indenting
     for ($i=0; $i < $lastlevel; ++$i) {
 	++$level;
 	if ($level <= $TLEVELS) {
-	    ($tmpl = $TINDENTBEG) =~ s/$VarExp/&replace_li_var($1,'')/geo;
+	    ($tmpl = $TSLICEINDENTBEG) =~ s/$VarExp/&replace_li_var($1,'')/geo;
 	    $slicetxt .= $tmpl;
 	}
     }
@@ -469,7 +497,7 @@ sub make_thread_slice {
 	if (($lastlevel > 0) && ($tlevel < $lastlevel)) {
 	    for ($i=$tlevel; $i < $lastlevel; ++$i) {
 		if ($level <= $TLEVELS) {
-		    ($tmpl = $TINDENTEND) =~
+		    ($tmpl = $TSLICEINDENTEND) =~
 			s/$VarExp/&replace_li_var($1,'')/geo;
 		    $slicetxt .= $tmpl;
 		}
@@ -477,27 +505,28 @@ sub make_thread_slice {
 	    }
 	    $lastlevel = $tlevel;
 	    if ($lastlevel < 1) {	# Check if continuation done
-		($tmpl = $TCONTEND) =~
+		($tmpl = $TSLICECONTEND) =~
 		    s/$VarExp/&replace_li_var($1,'')/geo;
 		$slicetxt .= $tmpl;
 	    }
 	}
 	unless ($Printed{$index} ||
 		($HasRef{$index} && $TVisible{$HasRef{$index}})) {
-	    $slicetxt .= &make_thread($index, ($lastlevel > 0) ? 0 : 1);
+	    $slicetxt .= &make_thread($index,
+			      (($lastlevel > 0) ? 0 : 1), $refindex);
 	}
     }
     # unindent if required
     for ($i=0; $i < $lastlevel; ++$i) {
 	if ($level <= $TLEVELS) {
-	    ($tmpl = $TINDENTEND) =~ s/$VarExp/&replace_li_var($1,'')/geo;
+	    ($tmpl = $TSLICEINDENTEND) =~ s/$VarExp/&replace_li_var($1,'')/geo;
 	    $slicetxt .= $tmpl;
 	}
 	--$level;
     }
     # close continuation if required
     if ($lastlevel > 0) {
-	($tmpl = $TCONTEND) =~ s/$VarExp/&replace_li_var($1,'')/geo;
+	($tmpl = $TSLICECONTEND) =~ s/$VarExp/&replace_li_var($1,'')/geo;
 	$slicetxt .= $tmpl;
     }
 
@@ -515,14 +544,16 @@ sub make_thread_slice {
 ##	Uses %Printed and $level defined by caller.
 ##
 sub make_thread {
-    local($idx, $top) = @_;
-    local(@repls, @srepls);
-    local($attop, $haverepls, $hvnirepls, $single, $depth, $i);
-    local($ret) = "";
+    my($idx, $top, $refidx) = @_;
+    my($attop, $haverepls, $hvnirepls, $single, $depth, $i);
+    my(@repls, @srepls) = ( );
+    my($ret) = "";
 
     ## Get replies
-    @repls  = sort increase_index split(/$bs/o, $Replies{$idx});
-    @srepls = sort increase_index split(/$bs/o, $SReplies{$idx});
+    @repls  = sort increase_index @{$Replies{$idx}}
+	if defined($Replies{$idx});
+    @srepls = sort increase_index @{$SReplies{$idx}}
+	if defined($SReplies{$idx});
     $depth  = $HasRefDepth{$idx};
     $hvnirepls = (@repls || @srepls);
 
@@ -541,20 +572,23 @@ sub make_thread {
     $single  = ($top && !$hvnirepls);
 
     if ($attop) {
-	$ret .= &expand_thread_var($idx, *TTOPBEG);
+	$ret .= &expand_thread_var($idx,
+		  ($idx eq $refidx) ? \$TSLICETOPBEGCUR : \$TSLICETOPBEG);
     } elsif ($single) {
-	$ret .= &expand_thread_var($idx, *TSINGLETXT);
+	$ret .= &expand_thread_var($idx,
+		  ($idx eq $refidx) ? \$TSLICESINGLETXTCUR: \$TSLICESINGLETXT);
     } else {
 	## Check for missing messages
 	if ($DoMissingMsgs) {
 	    for ($i = $depth; $i > 0; $i--) {
 		$level++;
-		$ret .= &expand_thread_var($idx, *TLINONE);
-		$ret .= &expand_thread_var($idx, *TSUBLISTBEG)
+		$ret .= &expand_thread_var($idx, \$TSLICELINONE);
+		$ret .= &expand_thread_var($idx, \$TSLICESUBLISTBEG)
 		    if $level <= $TLEVELS;
 	    }
 	}
-	$ret .= &expand_thread_var($idx, *TLITXT);
+	$ret .= &expand_thread_var($idx,
+		  ($idx eq $refidx) ? \$TSLICELITXTCUR : \$TSLICELITXT);
     }
 
     ## Increment level count if their are replies
@@ -567,20 +601,24 @@ sub make_thread {
 
     ## Print sub-threads
     if (@repls) {
-	$ret .= &expand_thread_var($idx, *TSUBLISTBEG)  if $level <= $TLEVELS;
+	$ret .= &expand_thread_var($idx, \$TSLICESUBLISTBEG)
+	    if $level <= $TLEVELS;
 	foreach (@repls) {
-	    $ret .= &make_thread($_);
+	    $ret .= &make_thread($_, 0, $refidx);
 	}
-	$ret .= &expand_thread_var($idx, *TSUBLISTEND)  if $level <= $TLEVELS;
+	$ret .= &expand_thread_var($idx, \$TSLICESUBLISTEND)
+	    if $level <= $TLEVELS;
     }
     if (@srepls) {
-	$ret .= &expand_thread_var($idx, *TSUBLISTBEG)  if $level <= $TLEVELS;
-	$ret .= &expand_thread_var($idx, *TSUBJECTBEG);
+	$ret .= &expand_thread_var($idx, \$TSLICESUBLISTBEG)
+	    if $level <= $TLEVELS;
+	$ret .= &expand_thread_var($idx, \$TSLICESUBJECTBEG);
 	foreach (@srepls) {
-	    $ret .= &make_thread($_);
+	    $ret .= &make_thread($_, 0, $refidx);
 	}
-	$ret .= &expand_thread_var($idx, *TSUBJECTEND);
-	$ret .= &expand_thread_var($idx, *TSUBLISTEND)  if $level <= $TLEVELS;
+	$ret .= &expand_thread_var($idx, \$TSLICESUBJECTEND);
+	$ret .= &expand_thread_var($idx, \$TSLICESUBLISTEND)
+	    if $level <= $TLEVELS;
     }
 
     ## Decrement level count if their were replies
@@ -590,8 +628,8 @@ sub make_thread {
     ## Check for missing messages
     if ($DoMissingMsgs && !($attop || $single)) {
 	for ($i = $depth; $i > 0; $i--) {
-	    $ret .= &expand_thread_var($idx, *TLINONEEND);
-	    $ret .= &expand_thread_var($idx, *TSUBLISTEND)
+	    $ret .= &expand_thread_var($idx, \$TSLICELINONEEND);
+	    $ret .= &expand_thread_var($idx, \$TSLICESUBLISTEND)
 		if $level <= $TLEVELS;
 	    $level--;
 	}
@@ -599,22 +637,23 @@ sub make_thread {
 
     ## Close entry text
     if ($attop) {
-	$ret .= &expand_thread_var($idx, *TTOPEND);
+	$ret .= &expand_thread_var($idx,
+		  ($idx eq $refidx) ? \$TSLICETOPENDCUR : \$TSLICETOPEND);
     } elsif (!$single) {
-	$ret .= &expand_thread_var($idx, *TLIEND);
+	$ret .= &expand_thread_var($idx,
+		  ($idx eq $refidx) ? \$TSLICELIENDCUR : \$TSLICELIEND);
     }
 
     $ret;
 }
 
 ##---------------------------------------------------------------------------
-##	Expand text based upon resource variable referenced by *tvar.
+##	Expand text based upon resource variable referenced by $tvar.
 ##
 sub expand_thread_var {
-    local($index, *tvar) = @_;
-    local($i_p0, $filename, $expstr, $msgnum);
-
-    ($expstr = $tvar) =~ s/$VarExp/&replace_li_var($1,$index)/geo;
+    my($index, $tvar) = @_;
+    my($expstr);
+    ($expstr = $$tvar) =~ s/$VarExp/&replace_li_var($1,$index)/geo;
     $expstr;
 }
 
