@@ -1,6 +1,6 @@
 ##---------------------------------------------------------------------------##
 ##  File:
-##	@(#) mhtxthtml.pl 2.15 01/06/10 17:36:57
+##	@(#) mhtxthtml.pl 2.17 01/08/26 02:10:34
 ##  Author:
 ##      Earl Hood       mhonarc@mhonarc.org
 ##  Description:
@@ -34,6 +34,7 @@ package m2h_text_html;
 
 # Beginning of URL match expression
 my $Url	= '(\w+://|\w+:)';
+
 # Script related attributes
 my $SAttr = q/(?:onload|onunload|onclick|ondblclick|/.
 	    q/onmouse(?:down|up|over|move|out)|/.
@@ -42,6 +43,13 @@ my $SAttr = q/(?:onload|onunload|onclick|ondblclick|/.
 my $SElem = q/(?:applet|base|embed|form|ilayer|input|layer|link|meta|object|/.
 	    q/option|param|select|textarea)/;
 
+# Elements with auto-loaded URL attributes
+my $AElem = q/(?:img|body|iframe|frame|object|script|input)/;
+	    # XXX: What about INS, DEL?
+# URL attributes
+my $UAttr = q/(?:href|src|background|classid|data|longdesc)/;
+	    # XXX: What about codebase, usemap?
+
 ##---------------------------------------------------------------------------
 ##	The filter must modify HTML content parts for merging into the
 ##	final filtered HTML messages.  Modification is needed so the
@@ -49,6 +57,24 @@ my $SElem = q/(?:applet|base|embed|form|ilayer|input|layer|link|meta|object|/.
 ##
 ##	Arguments:
 ##
+##	allowcomments	Preserve any comment declarations.  Normally
+##			Comment declarations are munged to prevent
+##			SSI attacks or comments that can conflict
+##			with MHonArc processing.  Use this option
+##			with care.
+##
+##	allownoncidurls	Preserve URL-based attributes that are not
+##			cid: URLs.  Normally, any URL-based attribute
+##			-- href, src, background, classid, data,
+##			longdesc -- will be stripped if it is not a
+##			cid: URL.  This is to prevent malicious URLs
+##			that verify mail addresses for spam purposes,
+##			secretly set cookies, or gather some
+##			statistical data automatically with the use of
+##			elements that cause browsers to automatically
+##			fetch data: IMG, BODY, IFRAME, FRAME, OBJECT,
+##			SCRIPT, INPUT.
+##
 ##	allowscript	Preserve any markup associated with scripting.
 ##			This includes elements and attributes related
 ##			to scripting.  The default is to delete any
@@ -56,34 +82,38 @@ my $SElem = q/(?:applet|base|embed|form|ilayer|input|layer|link|meta|object|/.
 ##
 ##	nofont  	Remove <FONT> tags.
 ##
-##	allowcomments	Preserve any comment declarations.  Normally
-##			Comment declarations are munged to prevent
-##			SSI attacks or comments that can conflict
-##			with MHonArc processing.  Use this option
-##			with care.
+##	notitle  	Do not print title.
 ##
 sub filter {
-    local($header, *fields, *data, $isdecode, $args) = @_;
+    my($fields, $data, $isdecode, $args) = @_;
     local(@files) = ();	# !!!Used by resolve_cid!!!
     my $base 	 = '';
     my $title	 = '';
     my $noscript = 1;
        $noscript = 0  if $args =~ /\ballowscript\b/i;
     my $nofont	 = $args =~ /\bnofont\b/i;
+    my $notitle	 = $args =~ /\bnotitle\b/i;
+    my $onlycid  = $args !~ /\ballownoncidurls\b/i;
     my $tmp;
 
     ## Check comment declarations: may screw-up mhonarc processing
     ## and avoids someone sneaking in SSIs.
-    #$data =~ s/<!(?:--(?:[^-]|-[^-])*--\s*)+>//go; # can crash perl
-    $data =~ s/<!--[^-]+[#X%\$\[]*/<!--/g;  # Just mung them (faster)
+    #$$data =~ s/<!(?:--(?:[^-]|-[^-])*--\s*)+>//go; # can crash perl
+    $$data =~ s/<!--[^-]+[#X%\$\[]*/<!--/g;  # Just mung them (faster)
 
     ## Get/remove title
-    if ($data =~ s|<title\s*>([^<]*)</title\s*>||io) {
-        $title = "<ADDRESS>Title: <STRONG>$1</STRONG></ADDRESS>\n";
+    if (!$notitle) {
+	if ($$data =~ s|<title\s*>([^<]*)</title\s*>||io) {
+	    $title = "<address>Title: <strong>$1</strong></address>\n"
+		unless $1 eq "";
+	}
+    } else {
+	$$data =~ s|<title\s*>[^<]*</title\s*>||io;
     }
+
     ## Get/remove BASE url
     BASEURL: {
-	if ($data =~ s|(<base\s[^>]*>)||i) {
+	if ($$data =~ s|(<base\s[^>]*>)||i) {
 	    $tmp = $1;
 	    if ($tmp =~ m|href\s*=\s*['"]([^'"]+)['"]|i) {
 		$base = $1;
@@ -92,37 +122,44 @@ sub filter {
 	    }
 	    last BASEURL  if ($base =~ /\S/);
 	} 
-	if ((defined($tmp = $fields{'content-base'}) ||
-	     defined($tmp = $fields{'content-location'})) && ($tmp =~ m%/%)) {
+	if ((defined($tmp = $fields->{'content-base'}[0]) ||
+	       defined($tmp = $fields->{'content-location'}[0])) &&
+	       ($tmp =~ m%/%)) {
 	    ($base = $tmp) =~ s/['"\s]//g;
 	}
     }
     $base =~ s|(.*/).*|$1|;
 
     ## Strip out certain elements/tags to support proper inclusion
-    $data =~ s|<!doctype\s[^>]*>||io;
-    $data =~ s|</?html\b[^>]*>||gio;
-    $data =~ s|</?x-html\b[^>]*>||gio;
-    $data =~ s|<head\s*>[\s\S]*</head\s*>||io;
+    $$data =~ s|<!doctype\s[^>]*>||io;
+    $$data =~ s|</?html\b[^>]*>||gio;
+    $$data =~ s|</?x-html\b[^>]*>||gio;
+    $$data =~ s|<head\s*>[\s\S]*</head\s*>||io;
 
     ## Strip out <font> tags if requested
     if ($nofont) {
-	$data =~ s|<style[^>]*>.*?</style\s*>||gios;
-	$data =~ s|</?font\b[^>]*>||gio;
+	$$data =~ s|<style[^>]*>.*?</style\s*>||gios;
+	$$data =~ s|</?font\b[^>]*>||gio;
     }
 
     ## Strip out scripting markup if requested
     if ($noscript) {
-	$data =~ s|<script[^>]*>.*?</script\s*>||gios;
-	$data =~ s|<style[^>]*>.*?</style\s*>||gios  unless $nofont;
-	$data =~ s|\b$SAttr\b\s*=\s*"[^"]*"||gio; #"
-	$data =~ s|\b$SAttr\b\s*=\s*'[^']*'||gio; #'
-	$data =~ s|\b$SAttr\b\s*=\s*[^\s>]+||gio;
-	$data =~ s|</?$SElem[^>]*>||gio;
+	$$data =~ s|<script[^>]*>.*?</script\s*>||gios;
+	$$data =~ s|<style[^>]*>.*?</style\s*>||gios  unless $nofont;
+	$$data =~ s|\b$SAttr\b\s*=\s*"[^"]*"||gio; #"
+	$$data =~ s|\b$SAttr\b\s*=\s*'[^']*'||gio; #'
+	$$data =~ s|\b$SAttr\b\s*=\s*[^\s>]+||gio;
+	$$data =~ s|</?$SElem[^>]*>||gio;
+    }
+    
+    if ($onlycid) {
+        $$data =~ s/($AElem\b[^>]+$UAttr\s*=\s*['"])([^'"]+)(['"])
+		   /&preserve_cid($1, $2, $3)
+		   /geoix;
     }
 
     ## Check for body attributes
-    if ($data =~ s|<body\b([^>]*)>||i) {
+    if ($$data =~ s|<body\b([^>]*)>||i) {
 	require 'mhutil.pl';
 	my $a = $1;
 	my %attr = mhonarc::parse_vardef_str($a, 1);
@@ -162,24 +199,24 @@ sub filter {
 		$tsuf .= '</font>';
 	    }
 	    $tsuf .= '</td></tr></table>';
-	    $data = $tpre . $data . $tsuf;
+	    $$data = $tpre . $$data . $tsuf;
 	}
     }
-    $data =~ s|</?body[^>]*>||ig;
+    $$data =~ s|</?body[^>]*>||ig;
 
     ## Modify relative urls to absolute using BASE
     if ($base =~ /\S/) {
-        $data =~ s/((?:href|src|background)\s*=\s*['"])([^'"]+)(['"])/
-		   join("", $1, &addbase($base,$2), $3)/geix;
+        $$data =~ s/($UAttr\s*=\s*['"])([^'"]+)(['"])/
+		   join("", $1, &addbase($base,$2), $3)/geoix;
     }
 
     ## Check for CID URLs (multipart/related HTML)
-    $data =~ s/((?:href|src|background)\s*=\s*['"])([^'"]+)(['"])/
-	       join("", $1, &resolve_cid($2), $3)/geix;
-    $data =~ s/((?:href|src|background)\s*=\s*)([^'">][^\s>]+)/
-	       join("", $1, '"', &resolve_cid($2), '"')/geix;
+    $$data =~ s/($UAttr\s*=\s*['"])([^'"]+)(['"])/
+	       join("", $1, &resolve_cid($2), $3)/geoix;
+    $$data =~ s/($UAttr\s*=\s*)([^'">][^\s>]+)/
+	       join("", $1, '"', &resolve_cid($2), '"')/geoix;
 
-    ($title.$data, @files);
+    ($title.$$data, @files);
 }
 
 ##---------------------------------------------------------------------------
@@ -220,26 +257,39 @@ sub addbase {
 sub resolve_cid {
     my $cid = shift;
     my $href = $readmail::Cid{$cid};
-    if (!defined($href)) { return ""; }
+    if (!defined($href)) { return ($cid =~ /^cid:/i)? "": $cid; }
 
     require 'mhmimetypes.pl';
     my $filename;
     my $decodefunc =
-	readmail::load_decoder($href->{'fields'}{'content-transfer-encoding'});
+	readmail::load_decoder(
+	    $href->{'fields'}{'content-transfer-encoding'}[0]);
     if (defined($decodefunc) && defined(&$decodefunc)) {
-	my $data = &$decodefunc($href->{'body'});
+	my $data = &$decodefunc(${$href->{'body'}});
 	$filename = mhonarc::write_attachment(
-			    $href->{'fields'}{'content-type'}, \$data);
+			    $href->{'fields'}{'content-type'}[0], \$data);
     } else {
 	$filename = mhonarc::write_attachment(
-			    $href->{'fields'}{'content-type'},
-			    \$href->{'body'});
+			    $href->{'fields'}{'content-type'}[0],
+			    $href->{'body'});
     }
     $href->{'filtered'} = 1; # mark part filtered for readmail.pl
-    push(@files, $filename); # @files defined in filter
+    push(@files, $filename); # @files defined in filter!!
     $filename;
 }
 
 ##---------------------------------------------------------------------------
 
+sub preserve_cid {
+    my $pre = shift;
+    my $url = shift;
+    my $post = shift;
+    if ($url =~ /^cid:/i) {
+	$pre . $url . $post;
+    } else {
+	$pre . 'javascript:void(0);' . $post;
+    }
+}
+##---------------------------------------------------------------------------
+
 1;
