@@ -1,6 +1,6 @@
 ##---------------------------------------------------------------------------##
 ##  File:
-##	@(#) mhutil.pl 2.6 99/08/08 20:01:54
+##	@(#) mhutil.pl 2.10 01/08/26 02:17:15
 ##  Author:
 ##      Earl Hood       mhonarc@pobox.com
 ##  Description:
@@ -117,9 +117,11 @@ sub sort_messages {
 
     } elsif ($subsort) {
 	## Subject order
-	my $hs = scalar(%Subject);  $hs =~ s|^[^/]/||;
 	my(%sub, $idx, $sub);
-	keys(%sub) = $hs;
+	eval {
+	    my $hs = scalar(%Subject);  $hs =~ s|^[^/]+/||;
+	    keys(%sub) = $hs;
+	};
 	while (($idx, $sub) = each(%Subject)) {
 	    $sub = lc $sub;
 	    1 while $sub =~ s/$SubReplyRxp//io;
@@ -127,7 +129,7 @@ sub sort_messages {
 	    $sub{$idx} = $sub;
 	}
 	if ($revsort) {
-	    return sort { ($sub{$b} cmp $sub{$a}) ||
+	    return sort { ($sub{$a} cmp $sub{$b}) ||
 			  (get_time_from_index($b) <=> get_time_from_index($a))
 			} keys %Subject;
 	} else {
@@ -138,15 +140,17 @@ sub sort_messages {
 	
     } elsif ($authsort) {
 	## Author order
-	my $hs = scalar(%From);  $hs =~ s|^[^/]/||;
 	my(%from, $idx, $from);
-	keys(%from) = $hs;
+	eval {
+	    my $hs = scalar(%From);  $hs =~ s|^[^/]+/||;
+	    keys(%from) = $hs;
+	};
 	while (($idx, $from) = each(%From)) {
 	    $from = lc extract_email_name($from);
 	    $from{$idx} = $from;
 	}
 	if ($revsort) {
-	    return sort { ($from{$b} cmp $from{$a}) ||
+	    return sort { ($from{$a} cmp $from{$b}) ||
 			  (get_time_from_index($b) <=> get_time_from_index($a))
 			} keys %Subject;
 	} else {
@@ -254,7 +258,8 @@ sub get_time_from_date {
     local($mday, $mon, $yr, $hr, $min, $sec, $zone) = @_;
     local($time) = 0;
 
-    $yr -= 1900  if $yr >= 1900;
+    $yr -= 1900  if $yr >= 1900;  # if given full 4 digit year
+    $yr += 100   if $yr <= 37;    # in case of 2 digit years
     if (($yr < 70) || ($yr > 137)) {
 	warn "Warning: Bad year (", $yr+1900, ") using current\n";
 	$yr = (localtime(time))[5];
@@ -266,15 +271,28 @@ sub get_time_from_date {
 	$time = &timegm($sec,$min,$hr,$mday,$mon,$yr);
 
 	# try to modify time/date based on timezone
-	if ($zone =~ /^[\+-]\d+$/) {	# numeric timezone
-	    $time -= &zone_offset_to_secs($zone);
+	OFFSET: {
+	    # numeric timezone
+	    if ($zone =~ /^[\+-]\d+$/) {
+		$time -= &zone_offset_to_secs($zone);
+		last OFFSET;
+	    }
+	    # Zone
+	    if (defined($Zone{$zone})) {
+		# timezone abbrev
+		$time += &zone_offset_to_secs($Zone{$zone});
+		last OFFSET;
 
-	# timezone abbrev
-	} elsif (defined($Zone{$zone})) {
-	    $time += &zone_offset_to_secs($Zone{$zone});
-
-	# undefined timezone
-	} else {
+	    }
+	    # Zone[+-]DDDD
+	    if ($zone =~ /^([A-Z]\w+)([\+-]\d+)$/) {
+		$time -= &zone_offset_to_secs($2);
+		if (defined($Zone{$1})) {
+		    $time += &zone_offset_to_secs($Zone{$1});
+		    last OFFSET;
+		}
+	    }
+	    # undefined timezone
 	    warn qq|Warning: Unrecognized time zone, "$zone"\n|;
 	}
 
@@ -314,32 +332,34 @@ sub get_header_tags {
 
 ##---------------------------------------------------------------------------
 ##	Format message headers in HTML.
+##	$html = htmlize_header($fields_hash_ref);
 ##
 sub htmlize_header {
-    local(*fields, *l2o) = @_;
+    my $fields = shift;
     my($key,
        $tago, $tagc,
        $ftago, $ftagc,
-       $mesg, $item,
-       @array, %hf);
-    local($tmp);
+       $item,
+       @array);
+    my($tmp);
 
-    $mesg = "";
-    %hf = %fields;
+    my $mesg = "";
+    my %hf = %$fields;
     foreach $item (@FieldOrder) {
 	if ($item eq '-extra-') {
 	    foreach $key (sort keys %hf) {
 		next  if $FieldODefs{$key};
+		next  if $key =~ /^x-mha-/;
 		delete $hf{$key}, next  if &exclude_field($key);
 
-		@array = split(/$readmail::FieldSep/o, $hf{$key});
+		@array = @{$hf{$key}};
 		foreach $tmp (@array) {
 		    $tmp = $HFieldsList{$key} ? mlist_field_add_links($tmp) :
 						&$MHeadCnvFunc($tmp);
-		    &field_add_links($key, *tmp);
-		    ($tago, $tagc, $ftago, $ftagc) = &get_header_tags($key);
+		    $tmp = field_add_links($key, $tmp, $fields);
+		    ($tago, $tagc, $ftago, $ftagc) = get_header_tags($key);
 		    $mesg .= join('', $LABELBEG,
-				  $tago, $l2o{$key}, $tagc, $LABELEND,
+				  $tago, ucfirst($key), $tagc, $LABELEND,
 				  $FLDBEG, $ftago, $tmp, $ftagc, $FLDEND,
 				  "\n");
 		}
@@ -347,14 +367,14 @@ sub htmlize_header {
 	    }
 	} else {
 	    if (!&exclude_field($item) && $hf{$item}) {
-		@array = (split(/$readmail::FieldSep/o, $hf{$item}));
+		@array = @{$hf{$item}};
 		foreach $tmp (@array) {
 		    $tmp = $HFieldsList{$item} ? mlist_field_add_links($tmp) :
 						 &$MHeadCnvFunc($tmp);
-		    &field_add_links($item, *tmp);
+		    $tmp = field_add_links($item, $tmp, $fields);
 		    ($tago, $tagc, $ftago, $ftagc) = &get_header_tags($item);
 		    $mesg .= join('', $LABELBEG,
-				  $tago, $l2o{$item}, $tagc, $LABELEND,
+				  $tago, ucfirst($item), $tagc, $LABELEND,
 				  $FLDBEG, $ftago, $tmp, $ftagc, $FLDEND,
 				  "\n");
 		}
@@ -388,12 +408,17 @@ sub mlist_field_add_links {
 ##
 sub field_add_links {
     my $label = lc shift;
-    local(*fld_text) = shift;
+    my $fld_text = shift;
+    my $fields	 = shift;
 
     LBLSW: {
 	if ($HFieldsAddr{$label}) {
 	    if (!$NOMAILTO) {
-		$fld_text =~ s|([\!\%\w\.\-+=/]+@[\w\.\-]+)|&mailUrl($1)|ge;
+		$fld_text =~ s|([\!\%\w\.\-+=/]+@[\w\.\-]+)
+			      |&mailUrl($1, $fields->{'x-mha-message-id'},
+					    $fields->{'x-mha-subject'},
+					    $fields->{'x-mha-from'});
+			      |gex;
 	    } else {
 		$fld_text =~ s|([\!\%\w\.\-+=/]+@[\w\.\-]+)
 			      |&htmlize(&rewrite_address($1))
@@ -402,11 +427,12 @@ sub field_add_links {
 	    last LBLSW;
 	}
 	if ($label eq 'newsgroup') {
-	    &newsurl(*fld_text)  unless $NONEWS;
+	    $fld_text = newsurl($fld_text)  unless $NONEWS;
 	    last LBLSW;
 	}
 	last LBLSW;
     }
+    $fld_text;
 }
 
 
@@ -414,26 +440,25 @@ sub field_add_links {
 ##	Routine to add news links of newsgroups names
 ##
 sub newsurl {
-    local(*str) = shift;
-    my(@groups) = ();
+    my $str = shift;
     my $h = "";
 
     if ($str =~ s/^([^:]*:\s*)//) {
 	$h = $1;
     }
     $str =~ s/\s//g;			# Strip whitespace
-    @groups = split(/,/, $str);		# Split groups
+    my @groups = split(/,/, $str);	# Split groups
     foreach (@groups) {			# Make hyperlinks
 	s|(.*)|<A HREF="news:$1">$1</A>|;
     }
-    $str = $h . join(', ', @groups);	# Rejoin string
+    $h . join(', ', @groups);	# Rejoin string
 }
 
 ##---------------------------------------------------------------------------
-##	$sub, $msgid, $from come from read_mail_header() (ugly!!!!)
+##	$html = mailUrl($email_addr, $msgid, $subject, $from);
 ##
 sub mailUrl {
-    my($eaddr) = shift;
+    my($eaddr, $msgid, $sub, $from) = @_;
 
     local $_;
     my($url) = ($MAILTOURL);
@@ -518,5 +543,49 @@ sub msgid_to_filename {
     $msgid;
 }
 
+##---------------------------------------------------------------------------##
+##	Check if new follow up list for a message is different from
+##	old follow up list.
+##
+sub is_follow_ups_diff {
+    my $f	= $Follow{$_[0]};
+    my $o	= $FollowOld{$_[0]};
+    if (defined($f) && defined($o)) {
+	return 1  unless @$f == @$o;
+	local $^W = 0;
+	my $i;
+	for ($i=0; $i < @$f; ++$i) {
+	    return 1  if $f->[$i] ne $o->[$i];
+	}
+	return 0;
+    }
+    return (defined($f) || defined($o));
+}
+
+##---------------------------------------------------------------------------##
+
+sub log_mesg {
+    my $fh	= shift;
+    my $doDate	= shift;
+
+    if ($doDate) {
+	my($sec,$min,$hour,$mday,$mon,$year) = localtime(time);
+	print $fh sprintf("[%4d-%02d-%02d %02d:%02d:%02d] ",
+			  $year+1900, $mon+1, $mday, $hour, $min, $sec);
+    }
+    print $fh @_;
+}
+
+##---------------------------------------------------------------------------##
+
+sub dump_hash {
+    my $fh = shift;
+    my $h = shift;
+    local $_;
+    foreach (sort keys %$h) {
+	print $fh "$_ => ", $h->{$_}, "\n";
+    }
+}
+
 ##---------------------------------------------------------------------------##
 1;
